;; Clay Servo - Copyright 2014 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Control a servo and two LED's embedded in a clay figure.

.include "tn85def.inc"
.avr8

; CKSEL = 0001
; CLKPS = 0000
; CKDIV8 = 0

; This should be default.. 1 MHz
; FUSE EXTD = 0xff
; FUSE HIGH = 0xdf
; FUSE LOW =  0x62

; r0  = 0
; r1  = 1
; r14 =
; r15 = 255
; r16 =
; r17 =
; r18 =
; r20 = interrupt counter for servo reset
; r21 = interrupt counter for servo position
; r22 =
; r23 =
; r24 =
; r25 = new servo value
; r26 =
; r27 =
; r30 =
; r31 =
;

.org 0x000
  rjmp start
.org 0x00a
  rjmp timer_interrupt

start:
  ;; Interrupts off
  cli

  ;; Set up stack ptr
  ;ldi r17, RAMEND>>8
  ;out SPH, r17
  ldi r17, RAMEND&255
  out SPL, r17

  ;; r0 = 0, r1 = 1, r15 = 255
  eor r0, r0
  eor r1, r1
  inc r1
  ldi r17, 0xff
  mov r15, r17

  ldi r20, 0
  ldi r25, 15 ; servo centered 

  ;; Set up PORTB
  ;; P0 = LED1
  ;; P1 = LED2
  ;; P2 = servo
  ldi r17, 7
  out DDRB, r17             ; PB0, PB1, PB2 are output
  ldi r17, 0
  out PORTB, r17            ; Everything off to start

  ;; Set up TIMER0
  ldi r17, 100                   ; 0.0001 seconds per interrupt
  out OCR0A, r17

  ldi r17, (1<<OCIE0A)
  out TIMSK, r17                 ; enable interrupt compare A 
  ldi r17, (1<<WGM01)
  out TCCR0A, r17                ; normal counting (0xffff is top, count up)
  ldi r17, (1<<CS00)             ; CTC OCR0A  Clear Timer on Compare
  out TCCR0B, r17                ; prescale = 1 from clock source

  ; Interrupts on
  sei

main:
  ldi r25, 15 ; servo centered 
  call long_delay

  ldi r25, 12 ; look left
  call short_delay
  sbi PORTB, 0
  sbi PORTB, 1
  call short_delay
  cbi PORTB, 0
  cbi PORTB, 1
  call short_delay

  ldi r25, 15 ; servo centered 
  call short_delay

  ldi r25, 17 ; look right
  call short_delay
  sbi PORTB, 0
  sbi PORTB, 1
  call short_delay
  cbi PORTB, 0
  cbi PORTB, 1
  call short_delay

  rjmp main

short_delay:
  ldi r24, 7
short_delay_repeat:
  call delay
  dec r24
  brne long_delay_repeat
  ret

long_delay:
  ldi r24, 7 * 10
long_delay_repeat:
  call delay
  dec r24
  brne long_delay_repeat
  ret

delay:
  ldi r23, 0
outer_loop:
  ldi r22, 0
inner_loop:
  dec r22
  brne inner_loop
  dec r23
  brne outer_loop
  ret

timer_interrupt:
  in r7, SREG
  inc r20
  cp r20, r21
  brne no_servo_change
  cbi PORTB, 2
no_servo_change:
  cpi r20, 200
  brne no_servo_reset
  sbi PORTB, 2
  ldi r20, 0
  mov r21, r25 
no_servo_reset:

exit_interrupt:
  out SREG, r7
  reti


signature:
.db "Clay Servo - Copyright 2014 - Michael Kohn - Version 0.01",0



